require "rails_admin_deposit/engine"

module RailsAdminDeposit
  # Your code goes here...
end

require 'rails_admin/config/actions'

module RailsAdmin
  module Config
    module Actions
      class Deposit < Base
        RailsAdmin::Config::Actions.register(self)
        
        
        register_instance_option :object_level do
          false
        end
        # This ensures the action only shows up for Users
        register_instance_option :visible? do
          # visible only if authorized and if the object has a defined method
          authorized? #&& bindings[:abstract_model].model == ChosenItem && bindings[:abstract_model].model.column_names.include?("barcode")
        end
        # We want the action on members, not the Users collection
        register_instance_option :member do
          false
        end

        register_instance_option :collection do
          false
        end
        register_instance_option :root? do
          true
        end

        register_instance_option :breadcrumb_parent do
          [nil]
        end

        register_instance_option :link_icon do
          'icon-barcode'
        end
        # You may or may not want pjax for your action
        register_instance_option :pjax? do
          true
        end
        
        register_instance_option :controller do
          Proc.new do # In modo che venga valutata ogni volta che viene chiamata
            if request.xhr? && !params["_pjax"]# Nel caso si voglia usare anche una interazione javascript
              if params[:what_to_do] == "data-action" && !params[:data].blank?
                # Here I have params[:data] and params[:barcode] set
                # in params[:data] I can find the not_before and not_after params
                # which I need
                expiration_date = DateTime.parse(params[:data][:not_after]).beginning_of_day unless params[:data][:not_after].blank?
                created_item = Item.create name: params[:data][:barcode], code: params[:data][:barcode], kind: Kind.find_by_code(params[:data][:barcode].split[0]), state: State.find_by(name: :deposit) do |i|
                  i.expiration_date = expiration_date unless expiration_date.blank?
                end
                if created_item.valid?
                  message = { message: I18n.t(:depositated, barcode: params[:data][:barcode]) }
                  message[:not_after] = expiration_date unless expiration_date.blank?
                  status = 200
                else
                  # Cannot create
                  message = { error: created_item.errors }
                  message[:not_after] = expiration_date unless expiration_date.blank?
                  status = 400
                end
              else
                if params[:barcode].blank?
                  message = { error: "#{I18n.t(:barcode_cannot_be_empty)}" }
                  status = 400
                else
                  item = Item.where(code: params[:barcode]).first
                  # Se il barcode esiste già come elemento, allora è un errore
                  # Perchè la fase di deposito è la prima, non deve esistere nel sistema
                  unless item.blank?
                    message = { error: "#{I18n.t(:item_already_exists)}: #{params[:barcode]}" }
                    status = 404
                  else
                    # Se non esiste, allora mando avanti il processo:
                    # - controllo che il barcode scansionato, contenga il codice di un kind di pasta conosciuto
                    unless Kind.pluck(:code).any? { |word| params[:barcode].include?(word) }
                      # --- Se non è così, allora errore
                      message = { error: "#{I18n.t(:barcode_not_contains_kind)}: #{params[:barcode]}" }
                      status = 404
                    else
                      # --- Se è così, continua:
                      # ----- ritorno ok e chiedo la data di expiration e di not before
                      # ----- a quel punto, dando ok, posso procedere al salvataggio
                      # not_before = DateTime.now + Settings.ns('phases_models').not_before.hours
                      not_after = params[:parameters][:not_after].presence || DateTime.now # + Settings.ns('phases_models').not_after.hours
                      message = { info: "#{I18n.t(:object_not_found_going_on)}", barcode: params[:barcode], not_after: not_after }
                      status = 200
                    end
                  end
                end
              end

              render json: MultiJson.dump(message), status: status
            end
          end
        end
      end
    end
  end
end

